///////////////////////////////////////////////////////////////////////////////
//
// wxFormBuilder - A Visual Dialog Editor for wxWidgets.
// Copyright (C) 2005 José Antonio Hurtado
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Written by
//   José Antonio Hurtado - joseantonio.hurtado@gmail.com
//   Juan Antonio Ortega  - jortegalalmolda@gmail.com
//
///////////////////////////////////////////////////////////////////////////////

#include "debug.h"
#include "rad/appdata.h"
#include <stdarg.h>

#include "wx/wx.h"

void Debug::Print( const wxChar* format, ... )
{
	#ifdef __WXFB_DEBUG__
        wxLog* target = AppData()->GetDebugLogTarget();
        if ( target != 0 )
        {
            wxLog* oldTarget = wxLog::SetActiveTarget( target );
            va_list argptr;
            va_start(argptr, format);

            wxString s;
            s.PrintfV(format, argptr);

            wxLogMessage(s);

            va_end(argptr);
            wxLog::SetActiveTarget( oldTarget );
        }
	#endif //__WXFB_DEBUG__
}
