project.name = "wxFormBuilder Plugins"

-- Add packages here.
dopackage( "additional" )
dopackage( "common" )
dopackage( "containers" )
dopackage( "layout" )
dopackage( "wxAdditions" )

-- Add dependency packages here.
dopackage( "../sdk/plugin_interface" )
dopackage( "../sdk/tinyxml" )
