///////////////////////////////////////////////////////////////////////////////
//
// wxFormBuilder - A Visual Dialog Editor for wxWidgets.
// Copyright (C) 2005 José Antonio Hurtado
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Written by
//   RJP Computing - rjpcomputing@gmail.com
//
///////////////////////////////////////////////////////////////////////////////
#ifndef __wxFBEventAdditionalPanel__
#define __wxFBEventAdditionalPanel__

/**
@file
Subclass of AdditionalPanel, which is generated by wxFormBuilder.
@todo Add your event handlers directly to this file.
*/

#include "event_GUI.h"

/** Implementing AdditionalPanel */
class wxFBEventAdditionalPanel : public AdditionalPanel
{
public:
	/** Constructor */
	wxFBEventAdditionalPanel( wxWindow* parent );
};

#endif // __wxFBEventAdditionalPanel__
